--**** A P P L I C A T I O N   N O T E *************************************
--*
--* Title:               AVR-FPGA Interface Design #6
--* Version:             1.1
--* Last Updated:        October 8, 2001
--* Target:              AT94K05/10/40
--*
--* Support Hotline:     +1 (408) 436-4119
--* Support Email:       fpslic@atmel.com
--*
--**************************************************************************

library ieee;
use ieee.std_logic_1164.all;

entity COUNTER3 is
    port (
            CLOCK   : in    std_logic;
            RESET   : in    std_logic;
            ENABLE  : in    std_logic;
            LOAD1   : in    std_logic;
            LOAD2   : in    std_logic;
            aWE     : in    std_logic;
            D1      : in    std_logic_vector(7 downto 0);
            D2      : in    std_logic_vector(7 downto 0);
            Q1      :   out std_logic_vector(7 downto 0);
            Q2      :   out std_logic_vector(7 downto 0);
            RCO1    :   out std_logic;
            RCO2    :   out std_logic;
            RAMADDR :   out std_logic_vector(15 downto 0);
            RAMCLK  :   out std_logic;
            WE      :   out std_logic
         );            
end COUNTER3;


architecture DATAFLOW of COUNTER3 is

    component COUNTER
        port (
                CLOCK   : in    std_logic;
                RESET   : in    std_logic;
                ENABLE  : in    std_logic;
                LOAD    : in    std_logic;
                aWE     : in    std_logic;
                D       : in    std_logic_vector(7 downto 0);
                Q       :   out std_logic_vector(7 downto 0);
                RCO     :   out std_logic
             );
    end component;

    component COUNTER2
        port (
                CLOCK   : in    std_logic;
                RESET   : in    std_logic;
                ENABLE  : in    std_logic;
                LOAD    : in    std_logic;
                aWE     : in    std_logic;
                D       : in    std_logic_vector(7 downto 0);
                Q       :   out std_logic_vector(7 downto 0);
                RCO     :   out std_logic
             );
    end component;

    signal rDATA : std_logic_vector(7 downto 0);

begin
    
    RAMCLK <= CLOCK;                                -- Assign SRAM Clock
    
    process (CLOCK)
    begin
        if CLOCK'event and CLOCK = '1' then
            RAMADDR <= "0000111110011100";          -- Assign SRAM Address = $0F9B
            WE <= '0';                              -- Disable SRAM Write Enable
            rDATA <= D2;                            -- Assign SRAM Data to rDATA
        end if;
    end process;

    C0 : COUNTER                                    -- Instantiate First Counter
        port map (
                    CLOCK   => CLOCK,
                    RESET   => RESET,
                    ENABLE  => ENABLE,
                    LOAD    => LOAD1,
                    aWE     => aWE,
                    D       => D1,
                    Q       => Q1,
                    RCO     => RCO1
                 );

    C1 : COUNTER2                                   -- Instantiate Second Counter
        port map (
                    CLOCK   => CLOCK,
                    RESET   => RESET,
                    ENABLE  => ENABLE,
                    LOAD    => LOAD2,
                    aWE     => aWE,
                    D       => rDATA,
                    Q       => Q2,
                    RCO     => RCO2
                 );

end DATAFLOW;